@@ -2,63 +2,25 @@ package Egg::Exception;
 #
 # Masatoshi Mizuno E<lt>lusheE<64>cpan.orgE<gt>
 #
-# $Id: Exception.pm 96 2007-05-07 21:31:53Z lushe $
+# $Id: Exception.pm 226 2008-01-27 10:23:16Z lushe $
 #
-
-=head1 NAME
-
-Egg::Exception - Exception module for Egg.
-
-=head1 SYNOPSIS
-
-  use Egg::Exception;
-  
-  Egg::Error->throw('The error occurs.');
-  
-  sub run {
-    my($e)= @_;
-    local $SIG{__DIE__}= sub { Egg::Error->throw(@_) };
-    eval {
-      $e->call_method;
-      };
-    if ($@) {
-      print STDERR "stack: ". $@->stacktrace;
-    }
-  }
-
-=head1 DESCRIPTION
-
-This is a module to treat the exception handling.
-
-=cut
 use strict;
+use warnings;
 
-our $VERSION= '2.00';
+our $VERSION= '3.00';
 
 package Egg::Error;
 use strict;
 use warnings;
 use Devel::StackTrace;
 use overload  '""' => 'stacktrace';
-use base qw/Class::Accessor::Fast/;
+use base qw/ Class::Accessor::Fast /;
 
-our $IGNORE_PACKAGE= [qw/main Class::C3 Carp NEXT/];
-our $IGNORE_CLASS  = [qw/Egg::Error/];
+our $IGNORE_PACKAGE= [qw/ main Carp /];
+our $IGNORE_CLASS  = [qw/ Egg::Error /];
 
 __PACKAGE__->mk_accessors(qw/ errstr frames as_string /);
 
-=head1 METHODS
-
-Egg::Exception doesn't have the method.
-Please call the method of Egg::Error.
-
-=head2 new
-
-Constructor.
-
-It processes it by L<Devel::StackTrace>.
-
-=cut
 sub new {
 	my $class = shift;
 	my $errstr= join '', @_;
@@ -81,28 +43,10 @@ sub new {
 	  frames   => [$stacktrace->frames],
 	  }, $class;
 }
-
-=head2 throw ( [MESSAGE] )
-
-After the exception message is thrown to the constructor, die is done.
-
-  Egg::Error->throw( 'internal error.' );
-
-=cut
 sub throw {
 	my $error= shift->new(@_);
 	die $error;
 }
-
-=head2 stacktrace
-
-The stack trace that has accumulated is output.
-
-  local $SIG{__DIE__}= sub { Egg::Error->throw(@_) };
-  eval{ ... code. };
-  if ($@) { die $@->stacktrace }
-
-=cut
 sub stacktrace {
 	my($self)= @_;
 	my @trace;
@@ -112,10 +56,77 @@ sub stacktrace {
 	"$self->{errstr} \n\n stacktrace: \n [". join("] \n [", @trace). "] \n";
 }
 
+1;
+
+__END__
+
+=head1 NAME
+
+Egg::Exception - The exception with stack trace is generated.
+
+=head1 SYNOPSIS
+
+  use Egg::Exception;
+  
+  Egg::Error->throw('The error occurs.');
+  
+  or
+  
+  local $SIG{__DIE__}= sub { Egg::Error->throw(@_) };
+  die 'The error occurs.';
+
+=head1 DESCRIPTION
+
+It is a module to vomit the message with stack trace when the exception is generated.
+
+=head1 METHODS
+
+=head2 new
+
+Constructor. This is internally called. 
+
+=head2 throw ([MESSAGE_STRING])
+
+After the constructor is let pass, the exception is generated.
+
+  Egg::Error->throw( 'internal error.' );
+
+=head2 stacktrace
+
+Only trace information on the object is returned.
+
+  local $SIG{__DIE__}= sub { Egg::Error->throw(@_) };
+  eval{ ... code. };
+  if ($@) { die $@->stacktrace }
+
+=head2 frames
+
+Trace information on the object is returned by the ARRAY reference.
+
+  local $SIG{__DIE__}= sub { Egg::Error->throw(@_) };
+  eval{ ... code. };
+  if ($@) { die join "\n", @{$@->frames} }
+
+=head2 as_string
+
+as_string of L<Devel::StackTrace > is returned.
+
+  local $SIG{__DIE__}= sub { Egg::Error->throw(@_) };
+  eval{ ... code. };
+  if ($@) { die $@->as_string }
+
+=head2 errstr
+
+Only the exception message of the object is returned.
+
+  local $SIG{__DIE__}= sub { Egg::Error->throw(@_) };
+  eval{ ... code. };
+  if ($@) { die $@->errstr }
+
 =head1 SEE ALSO
 
-L<Devel::StackTrace>,
 L<Egg::Release>,
+L<Devel::StackTrace>,
 
 =head1 AUTHOR
 
@@ -123,7 +134,7 @@ Masatoshi Mizuno, E<lt>lusheE<64>cpan.orgE<gt>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2007 Bee Flag, Corp. E<lt>L<http://egg.bomcity.com/>E<gt>, All Rights Reserved.
+Copyright (C) 2008 Bee Flag, Corp. E<lt>L<http://egg.bomcity.com/>E<gt>, All Rights Reserved.
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself, either Perl version 5.8.6 or,
@@ -131,4 +142,3 @@ at your option, any later version of Perl 5 you may have available.
 
 =cut
 
-1;
